@@ -26,6 +26,9 @@
 
 #include "gr-about-dialog.h"
 #include "gr-utils.h"
+#include "gr-recipe-store.h"
+
+#include <stdlib.h>
 
 #ifdef GDK_WINDOWING_QUARTZ
 #include <sys/sysctl.h>
@@ -805,6 +808,17 @@ G_GNUC_END_IGNORE_DEPRECATIONS
         }
 }
 
+static int
+compare_strings (gconstpointer a,
+                 gconstpointer b,
+                 gpointer      data)
+{
+        const char * const *sa = a;
+        const char * const *sb = b;
+
+        return strcmp (*sa, *sb);
+}
+
 GrAboutDialog *
 gr_about_dialog_new (void)
 {
@@ -820,26 +834,15 @@ gr_about_dialog_new (void)
                 "Nirbheek Chauhan",
                 "Ekta Nandwani",
                 "Maithili Bhide",
+                "Paxana Amanda Xander",
                 NULL
         };
-        const char *recipe_authors[] = {
-                "Ray Strode",
-                "Bastian Ilsø",
-                "Frederik Fyksen",
-                "Matthias Clasen",
-                "Allan Day",
-                "Erusan",
-                "Link Dupont",
-                "Tuomas Kuosmanen",
-                "Matthew Leeds",
-                "Alexandre Franke",
-                "Adrià Arrufat",
-                "Emmanuele Bassi",
-                "Adelia",
-                "Sylvia",
-                NULL
-        };
+        g_autofree char **recipe_authors = NULL;
+        guint length;
         g_autoptr(GdkPixbuf) logo = NULL;
+        const char *p;
+        const char *version;
+        GrRecipeStore *store;
 
         logo = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
                                          "org.gnome.Recipes",
@@ -847,13 +850,15 @@ gr_about_dialog_new (void)
                                          GTK_ICON_LOOKUP_FORCE_SIZE,
                                          NULL);
 
+        p = strrchr (PACKAGE_VERSION, '.');
+        if (p && (atoi (p + 1) % 2 == 1))
+                version = COMMIT_ID;
+        else
+                version = PACKAGE_VERSION;
+
         about = g_object_new (GR_TYPE_ABOUT_DIALOG,
                               "program-name", _("Recipes"),
-#if MICRO_VERSION % 2 == 1
-                              "version", COMMIT_ID,
-#else
-                              "version", PACKAGE_VERSION,
-#endif
+                              "version", version,
                               "copyright", "© 2016, 2017 Matthias Clasen",
                               "license-type", GTK_LICENSE_GPL_3_0,
                               "comments", _("GNOME loves to cook"),
@@ -864,10 +869,18 @@ gr_about_dialog_new (void)
                               "website-label", _("Learn more about Recipes"),
                               NULL);
 
+        store = gr_recipe_store_get ();
+        recipe_authors = gr_recipe_store_get_contributors (store, &length);
+        g_qsort_with_data (recipe_authors, length, sizeof (char *), compare_strings, NULL);
         gtk_about_dialog_add_credit_section (GTK_ABOUT_DIALOG (about),
-                                             _("Recipes by"), recipe_authors);
+                                             _("Recipes by"), (const char **)recipe_authors);
+
         add_built_logo (about);
         add_system_tab (about);
 
+	gtk_widget_realize (GTK_WIDGET (about));
+	gdk_window_set_functions (gtk_widget_get_window (GTK_WIDGET (about)),
+                                  GDK_FUNC_ALL | GDK_FUNC_MINIMIZE | GDK_FUNC_MAXIMIZE);
+
         return about;
 }
