@@ -22,7 +22,6 @@
 
 #include "gr-account.h"
 #include "gr-recipe-store.h"
-#include "gr-app.h"
 #include "gr-utils.h"
 
 #include <glib/gi18n.h>
@@ -105,7 +104,7 @@ account_response (GDBusConnection *connection,
         else {
                 g_autoptr(GError) error = NULL;
 
-                g_message ("Got an error from the Account portal");
+                g_info ("Got an error from the Account portal");
                 g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
                              _("Got an error from Account portal"));
 
@@ -128,7 +127,7 @@ window_handle_exported (GtkWindow  *window,
 
         cbdata->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
 	if (!cbdata->connection) {
-                g_message ("Could not talk to D-Bus: %s", error->message);
+                g_info ("Could not talk to D-Bus: %s", error->message);
                 cbdata->callback (NULL, NULL, NULL, cbdata->data, error);
                 free_callback_data (cbdata);
                 return;
@@ -150,7 +149,7 @@ window_handle_exported (GtkWindow  *window,
                                            &error);
 
         if (!ret) {
-                g_message ("Could not talk to Account portal: %s", error->message);
+                g_info ("Could not talk to Account portal: %s", error->message);
                 cbdata->callback (NULL, NULL, NULL, cbdata->data, error);
                 free_callback_data (cbdata);
                 return;
@@ -205,10 +204,10 @@ got_account_info (const char  *id,
         g_autoptr(GrChef) chef = NULL;
         g_autoptr(GError) local_error = NULL;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         if (error) {
-                g_message ("Failed to get account information: %s", error->message);
+                g_info ("Failed to get account information: %s", error->message);
 		id = g_get_user_name ();
 		name = g_get_real_name ();
 		image_path = NULL;
@@ -238,7 +237,7 @@ gr_ensure_user_chef (GtkWindow        *window,
         g_autoptr(GrChef) chef = NULL;
         UserChefData *cbdata;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         chef = gr_recipe_store_get_chef (store, gr_recipe_store_get_user_key (store));
         if (chef) {
