@@ -20,6 +20,8 @@
 
 #include "config.h"
 
+#include <stdlib.h>
+
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
@@ -27,7 +29,6 @@
 #include "gr-app.h"
 #include "gr-window.h"
 #include "gr-chef-dialog.h"
-#include "gr-recipe-store.h"
 #include "gr-cuisine.h"
 #include "gr-shell-search-provider.h"
 #include "gr-utils.h"
@@ -38,7 +39,6 @@ struct _GrApp
 {
         GtkApplication parent_instance;
 
-        GrRecipeStore *store;
         GrShellSearchProvider *search_provider;
         GrRecipeExporter *exporter;
 
@@ -53,7 +53,6 @@ gr_app_finalize (GObject *object)
 {
         GrApp *self = GR_APP (object);
 
-        g_clear_object (&self->store);
         g_clear_object (&self->search_provider);
         g_clear_object (&self->css_provider);
 
@@ -86,7 +85,7 @@ timer_expired (GSimpleAction *action,
 
         gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
-        recipe = gr_recipe_store_get_recipe (app->store, id);
+        recipe = gr_recipe_store_get_recipe (gr_recipe_store_get (), id);
         gr_window_timer_expired (GR_WINDOW (win), recipe, step);
 }
 
@@ -97,6 +96,7 @@ chef_information_activated (GSimpleAction *action,
 {
         GtkWindow *win;
 
+        gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
         gr_window_show_my_chef_information (GR_WINDOW (win));
 }
@@ -108,6 +108,7 @@ about_activated (GSimpleAction *action,
 {
         GtkWindow *win;
 
+        gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
         gr_window_show_about_dialog (GR_WINDOW (win));
 }
@@ -119,6 +120,7 @@ report_issue_activated (GSimpleAction *action,
 {
         GtkWindow *win;
 
+        gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
         gr_window_show_report_issue (GR_WINDOW (win));
 }
@@ -130,6 +132,7 @@ news_activated (GSimpleAction *action,
 {
         GtkWindow *win;
 
+        gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
         gr_window_show_news (GR_WINDOW (win));
 }
@@ -139,7 +142,11 @@ quit_activated (GSimpleAction *action,
                 GVariant      *parameter,
                 gpointer       app)
 {
-        g_application_quit (G_APPLICATION (app));
+        GtkWindow *win;
+
+        gr_app_activate (G_APPLICATION (app));
+        win = gtk_application_get_active_window (GTK_APPLICATION (app));
+        gtk_window_close (win);
 }
 
 static void
@@ -149,8 +156,8 @@ import_activated (GSimpleAction *action,
 {
         GtkWindow *win;
 
+        gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
-        gtk_window_present (win);
         gr_window_load_recipe (GR_WINDOW (win), NULL);
 }
 
@@ -161,8 +168,8 @@ export_activated (GSimpleAction *action,
 {
         GtkWindow *win;
 
+        gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
-        gtk_window_present (win);
         gr_window_save_all (GR_WINDOW (win));
 }
 
@@ -180,7 +187,7 @@ details_activated (GSimpleAction *action,
 
         gr_app_activate (G_APPLICATION (app));
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
-        recipe = gr_recipe_store_get_recipe (app->store, id);
+        recipe = gr_recipe_store_get_recipe (gr_recipe_store_get (), id);
         gr_window_show_recipe (GR_WINDOW (win), recipe);
 }
 
@@ -195,11 +202,75 @@ search_activated (GSimpleAction *action,
 
         g_variant_get (parameter, "&s", &search);
 
-        win = gtk_application_get_active_window (GTK_APPLICATION (app));
         gr_app_activate (G_APPLICATION (app));
+        win = gtk_application_get_active_window (GTK_APPLICATION (app));
         gr_window_show_search (GR_WINDOW (win), search);
 }
 
+#define DEFAULT_LEVELS (G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_LEVEL_MESSAGE)
+#define INFO_LEVELS (G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG)
+
+static gboolean verbose_logging;
+
+/* verbose_logging turns enables DEBUG and INFO messages just for our log domain.
+ * We also respect the G_MESSAGES_DEBUG environment variable.
+ */
+static GLogWriterOutput
+log_writer (GLogLevelFlags   log_level,
+            const GLogField *fields,
+            gsize            n_fields,
+            gpointer         user_data)
+{
+        if (!(log_level & DEFAULT_LEVELS)) {
+                const gchar *domains, *log_domain = NULL;
+                gsize i;
+
+                domains = g_getenv ("G_MESSAGES_DEBUG");
+
+                if (verbose_logging && domains == NULL)
+                        domains = G_LOG_DOMAIN;
+
+                if ((log_level & INFO_LEVELS) == 0 || domains == NULL)
+                        return G_LOG_WRITER_HANDLED;
+
+                for (i = 0; i < n_fields; i++) {
+                        if (g_strcmp0 (fields[i].key, "GLIB_DOMAIN") == 0) {
+                                log_domain = fields[i].value;
+                                break;
+                        }
+                }
+
+                if (!verbose_logging || strcmp (log_domain, G_LOG_DOMAIN) != 0) {
+                        if (strcmp (domains, "all") != 0 &&
+                            (log_domain == NULL || !strstr (domains, log_domain)))
+                                return G_LOG_WRITER_HANDLED;
+                }
+        }
+
+        if (g_log_writer_is_journald (fileno (stderr)) &&
+            g_log_writer_journald (log_level, fields, n_fields, user_data) == G_LOG_WRITER_HANDLED)
+                goto handled;
+
+        if (g_log_writer_standard_streams (log_level, fields, n_fields, user_data) == G_LOG_WRITER_HANDLED)
+                goto handled;
+
+        return G_LOG_WRITER_UNHANDLED;
+
+handled:
+        if (log_level & G_LOG_LEVEL_ERROR)
+                g_abort ();
+
+        return G_LOG_WRITER_HANDLED;
+}
+
+static void
+verbose_logging_activated (GSimpleAction *action,
+                           GVariant      *parameter,
+                           gpointer       application)
+{
+        g_variant_get (parameter, "b", &verbose_logging);
+}
+
 static GActionEntry app_entries[] =
 {
         { "timer-expired", timer_expired, "(si)", NULL, NULL },
@@ -211,16 +282,26 @@ static GActionEntry app_entries[] =
         { "details", details_activated, "(ss)", NULL, NULL },
         { "search", search_activated, "s", NULL, NULL },
         { "quit", quit_activated, NULL, NULL, NULL },
-        { "report-issue", report_issue_activated, NULL, NULL, NULL }
+        { "report-issue", report_issue_activated, NULL, NULL, NULL },
+        { "verbose-logging", verbose_logging_activated, "b", NULL, NULL }
 };
 
 static void
 load_application_css (GrApp *app)
 {
         gboolean dark;
-        const char *css_file;
-        const char *src_file;
-        const char *resource;
+        const char *css_file[2] = {
+                "recipes-light.css",
+                "recipes-dark.css"
+        };
+        const char *src_file[2] = {
+                "src/recipes-light.css",
+                "src/recipes-dark.css"
+        };
+        const char *resource[2] = {
+                "resource:///org/gnome/Recipes/recipes-light.css",
+                "resource:///org/gnome/Recipes/recipes-dark.css"
+        };
         const char *path;
         g_autofree char *css = NULL;
 
@@ -234,25 +315,15 @@ load_application_css (GrApp *app)
                       "gtk-application-prefer-dark-theme", &dark,
                       NULL);
 
-        if (dark) {
-                css_file = "recipes-dark.css";
-                src_file = "src/recipes-dark.css";
-                resource = "resource:///org/gnome/Recipes/recipes-dark.css";
-        }
-        else {
-                css_file = "recipes-light.css";
-                src_file = "src/recipes-light.css";
-                resource = "resource:///org/gnome/Recipes/recipes-light.css";
-        }
-
-        if (g_file_test (css_file, G_FILE_TEST_EXISTS))
-                path = css_file;
-        else if (g_file_test (src_file, G_FILE_TEST_EXISTS))
-                path = src_file;
+        /* Let designers test css tweaks in the build tree without rebuilding */
+        if (g_file_test (css_file[dark], G_FILE_TEST_EXISTS))
+                path = css_file[dark];
+        else if (g_file_test (src_file[dark], G_FILE_TEST_EXISTS))
+                path = src_file[dark];
         else
-                path = resource;
+                path = resource[dark];
 
-        g_message ("Loading application CSS from %s", path);
+        g_info ("Loading application CSS from %s", path);
 
         css = gr_cuisine_get_css (path);
 
@@ -265,7 +336,16 @@ load_application_css (GrApp *app)
 static void
 gr_app_startup (GApplication *app)
 {
-        const gchar *quit_accels[2] = { "<Ctrl>Q", NULL };
+	struct {
+		const char *detailed_action;
+		const char *accelerators[2];
+	} accels[] = {
+		{ "app.quit", { "<Primary>q", NULL } },
+		{ "app.search('')", { "<Primary>f", NULL } },
+		{ "win.copy", { "<Primary>c", NULL } },
+		{ "win.paste", { "<Primary>v", NULL } },
+	};
+	int i;
 
         G_APPLICATION_CLASS (gr_app_parent_class)->startup (app);
 
@@ -282,9 +362,26 @@ gr_app_startup (GApplication *app)
         }
 #endif
 
-        gtk_application_set_accels_for_action (GTK_APPLICATION (app),
-                                               "app.quit",
-                                               quit_accels);
+	for (i = 0; i < G_N_ELEMENTS (accels); i++) {
+        	gtk_application_set_accels_for_action (GTK_APPLICATION (app),
+						       accels[i].detailed_action,
+						       accels[i].accelerators);
+        }
+
+        {
+                g_autoptr(GtkBuilder) builder = NULL;
+                GObject *menu;
+
+                builder = gtk_builder_new_from_resource ("/org/gnome/Recipes/menus.ui");
+                if (strcmp (G_OBJECT_TYPE_NAME (gdk_display_get_default ()), "GdkQuartzDisplay") == 0) {
+                        menu = gtk_builder_get_object (builder, "menubar");
+                        gtk_application_set_menubar (GTK_APPLICATION (app), G_MENU_MODEL (menu));
+                }
+                else {
+                        menu = gtk_builder_get_object (builder, "app-menu");
+                        gtk_application_set_app_menu (GTK_APPLICATION (app), G_MENU_MODEL (menu));
+                }
+         }
 
         load_application_css (GR_APP (app));
 }
@@ -320,7 +417,7 @@ gr_app_dbus_register (GApplication    *application,
         GrApp *app = GR_APP (application);
 
         app->search_provider = gr_shell_search_provider_new ();
-        gr_shell_search_provider_setup (app->search_provider, app->store);
+        gr_shell_search_provider_setup (app->search_provider, gr_recipe_store_get ());
 
         return gr_shell_search_provider_register (app->search_provider, connection, error);
 }
@@ -341,7 +438,43 @@ gr_app_dbus_unregister (GApplication    *application,
 static void
 gr_app_init (GrApp *self)
 {
-        self->store = gr_recipe_store_new ();
+        g_application_add_main_option (G_APPLICATION (self),
+                                       "version", 'v',
+                                       G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
+                                       _("Print the version and exit"), NULL);
+        g_application_add_main_option (G_APPLICATION (self),
+                                       "verbose", 0,
+                                       G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
+                                       _("Turn on verbose logging"), NULL);
+
+        g_log_set_writer_func (log_writer, NULL, NULL);
+}
+
+static int
+gr_app_handle_local_options (GApplication *app,
+                             GVariantDict *options)
+{
+        gboolean value;
+
+        if (g_variant_dict_lookup (options, "version", "b", &value)) {
+                g_print ("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
+                return 0;
+        }
+
+        if (g_variant_dict_lookup (options, "verbose", "b", &value)) {
+                g_autoptr(GError) error = NULL;
+
+                if (!g_application_register (app, NULL, &error)) {
+                        g_printerr ("Failed to register: %s\n", error->message);
+                        return 1;
+                }
+
+                g_action_group_activate_action (G_ACTION_GROUP (app),
+                                                "verbose-logging",
+                                                g_variant_new_boolean (TRUE));
+        }
+
+        return -1;
 }
 
 static void
@@ -354,6 +487,7 @@ gr_app_class_init (GrAppClass *klass)
 
         application_class->startup = gr_app_startup;
         application_class->activate = gr_app_activate;
+        application_class->handle_local_options = gr_app_handle_local_options;
         application_class->open = gr_app_open;
         application_class->dbus_register = gr_app_dbus_register;
         application_class->dbus_unregister = gr_app_dbus_unregister;
@@ -367,9 +501,3 @@ gr_app_new (void)
                              "flags", G_APPLICATION_HANDLES_OPEN | G_APPLICATION_CAN_OVERRIDE_APP_ID,
                              NULL);
 }
-
-GrRecipeStore *
-gr_app_get_recipe_store (GrApp *app)
-{
-        return app->store;
-}
