@@ -33,7 +33,6 @@
 #include "gr-edit-page.h"
 #include "gr-recipe-store.h"
 #include "gr-recipe.h"
-#include "gr-app.h"
 #include "gr-utils.h"
 #include "gr-ingredients-list.h"
 #include "gr-cuisine.h"
@@ -59,6 +58,7 @@ struct _GrEditPage
 
         GrRecipe *recipe;
 
+        GtkWidget *main_content;
         GtkWidget *error_revealer;
         GtkWidget *error_label;
         GtkWidget *name_label;
@@ -137,10 +137,19 @@ struct _GrEditPage
         guint index_handler_id;
 
         char *author;
+        gboolean unsaved;
 };
 
 G_DEFINE_TYPE (GrEditPage, gr_edit_page, GTK_TYPE_BOX)
 
+enum {
+        PROP_0,
+        PROP_UNSAVED,
+        N_PROPS
+};
+
+static GParamSpec *props [N_PROPS];
+
 static char *get_text_view_text (GtkTextView *textview);
 static void  set_text_view_text (GtkTextView *textview,
                                  const char  *text);
@@ -152,6 +161,7 @@ dismiss_error (GrEditPage *page)
 }
 
 static void add_image_cb (GrEditPage *page);
+static void set_unsaved (GrEditPage *page);
 
 static void
 populate_image_flowbox (GrEditPage *page)
@@ -225,6 +235,7 @@ set_default_image_cb (GrEditPage *page)
 
         if (page->recipe)
                 g_object_set (page->recipe, "default-image", index, NULL);
+        set_unsaved (page);
 }
 
 static void
@@ -245,6 +256,7 @@ static void
 add_image_cb (GrEditPage *page)
 {
         gr_image_viewer_add_image (GR_IMAGE_VIEWER (page->images));
+        set_unsaved (page);
 }
 
 static char *
@@ -312,18 +324,21 @@ remove_image_cb (GrEditPage *page)
         instructions = get_text_view_text (GTK_TEXT_VIEW (page->instructions_field));
         rewritten = rewrite_instructions_for_removed_image (instructions, index);
         set_text_view_text (GTK_TEXT_VIEW (page->instructions_field), rewritten);
+        set_unsaved (page);
 }
 
 static void
 rotate_image_left_cb (GrEditPage *page)
 {
         gr_image_viewer_rotate_image (GR_IMAGE_VIEWER (page->images), 90);
+        set_unsaved (page);
 }
 
 static void
 rotate_image_right_cb (GrEditPage *page)
 {
         gr_image_viewer_rotate_image (GR_IMAGE_VIEWER (page->images), 270);
+        set_unsaved (page);
 }
 
 static void
@@ -442,6 +457,7 @@ remove_ingredient (GtkButton *button, GrEditPage *page)
                 page->active_row = NULL;
 
         gtk_widget_destroy (row);
+        set_unsaved (page);
 }
 
 static void
@@ -688,6 +704,7 @@ add_ingredient2 (GtkButton *button, GrEditPage *page)
                                  gtk_entry_get_text (GTK_ENTRY (page->new_ingredient_unit)),
                                  &amount, &unit);
         update_ingredient_row (row, amount, unit, ingredient);
+        set_unsaved (page);
 }
 
 static char *
@@ -1239,6 +1256,7 @@ add_step (GtkButton *button, GrEditPage *self)
         gtk_text_buffer_get_end_iter (buffer, &end);
         gtk_text_buffer_place_cursor (buffer, &end);
         gtk_text_buffer_insert_at_cursor (buffer, "\n\n", 2);
+        set_unsaved (self);
 }
 
 static void update_author_label (GrEditPage *page,
@@ -1260,16 +1278,14 @@ edit_chef (GrEditPage *page)
 {
         GrChefDialog *dialog;
         GtkWidget *win;
-        const char *author;
         GrRecipeStore *store;
         g_autoptr(GrChef) chef = NULL;
 
         win = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
-        author = page->author;
-        chef = gr_recipe_store_get_chef (store, author ? author : "");
+        chef = gr_recipe_store_get_chef (store, page->author ? page->author : "");
 
         dialog = gr_chef_dialog_new (chef, TRUE);
         g_signal_connect (dialog, "done", G_CALLBACK (chef_done), page);
@@ -1280,6 +1296,8 @@ edit_chef (GrEditPage *page)
         return TRUE;
 }
 
+
+
 static void
 gr_edit_page_init (GrEditPage *page)
 {
@@ -1339,7 +1357,7 @@ popover_keypress_handler (GtkWidget  *widget,
                         }
                 }
         }
-
+        
         return GDK_EVENT_PROPAGATE;
 }
 
@@ -1370,7 +1388,7 @@ preview_visible_changed (GrEditPage *page)
                 gtk_widget_set_sensitive (page->temperature_button, TRUE);
                 gtk_widget_set_visible (page->prev_step_button, FALSE);
                 gtk_widget_set_visible (page->next_step_button, FALSE);
-                gr_cooking_view_stop (GR_COOKING_VIEW (page->cooking_view));
+                gr_cooking_view_stop (GR_COOKING_VIEW (page->cooking_view), TRUE);
         }
         else {
                 g_autoptr(GArray) images = NULL;
@@ -1407,6 +1425,54 @@ next_step (GrEditPage *page)
         update_steppers (page);
 }
 
+static void
+gr_edit_page_set_property (GObject      *object,
+                           guint         prop_id,
+                           const GValue *value,
+                           GParamSpec   *pspec)
+{    
+        GrEditPage *self = GR_EDIT_PAGE (object);
+        switch (prop_id) {
+        case PROP_UNSAVED:
+                self->unsaved = g_value_get_boolean (value);
+                break;
+        default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        }
+}
+
+
+static void
+gr_edit_page_get_property (GObject    *object,
+                              guint       prop_id,
+                              GValue     *value,
+                              GParamSpec *pspec)
+{
+        GrEditPage *self = GR_EDIT_PAGE (object);
+
+        switch (prop_id) {
+        case PROP_UNSAVED:
+                g_value_set_boolean (value, self->unsaved);
+                break;
+        default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        }
+}
+
+static void
+set_unsaved (GrEditPage *page)
+{
+        g_object_set (G_OBJECT (page),"unsaved", TRUE, NULL);
+}
+
+static void
+gr_edit_page_grab_focus (GtkWidget *widget)
+{
+        GrEditPage *self = GR_EDIT_PAGE (widget);
+
+        gtk_widget_grab_focus (self->name_entry);
+}
+
 static void
 gr_edit_page_class_init (GrEditPageClass *klass)
 {
@@ -1414,9 +1480,19 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
         object_class->finalize = edit_page_finalize;
+        object_class->set_property = gr_edit_page_set_property;
+        object_class->get_property = gr_edit_page_get_property;
+
+        widget_class->grab_focus = gr_edit_page_grab_focus;
+
+        props [PROP_UNSAVED] = g_param_spec_boolean ("unsaved",
+                                                      NULL, NULL,
+                                                      TRUE, G_PARAM_READWRITE);
+        g_object_class_install_properties (object_class, N_PROPS, props);
 
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Recipes/gr-edit-page.ui");
 
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, main_content);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, error_revealer);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, error_label);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, name_label);
@@ -1512,6 +1588,7 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, preview_visible_changed);
         gtk_widget_class_bind_template_callback (widget_class, prev_step);
         gtk_widget_class_bind_template_callback (widget_class, next_step);
+        gtk_widget_class_bind_template_callback (widget_class, set_unsaved);
 }
 
 GtkWidget *
@@ -1650,6 +1727,7 @@ add_ingredients_segment (GrEditPage *page,
         gtk_widget_show (entry);
         gtk_entry_set_placeholder_text (GTK_ENTRY (entry), _("Name of the List"));
         gtk_entry_set_text (GTK_ENTRY (entry), segment_label[0] ? segment_label : "");
+        g_signal_connect_swapped (entry, "notify::text", G_CALLBACK (set_unsaved),page);
 
 #if defined(ENABLE_GSPELL) && defined(GSPELL_TYPE_ENTRY)
         {
@@ -1741,6 +1819,7 @@ remove_list (GtkButton *button, GrEditPage *page)
 
         update_segments (page);
         set_active_row (page, NULL);
+        set_unsaved (page);
 }
 
 static void
@@ -1770,9 +1849,7 @@ populate_ingredients (GrEditPage *page,
 {
         g_autoptr(GrIngredientsList) ingredients = NULL;
         g_autofree char **segs = NULL;
-        g_auto(GStrv) ings = NULL;
         int i, j;
-        GtkWidget *list;
         GtkWidget *button;
 
         container_remove_all (GTK_CONTAINER (page->ingredients_box));
@@ -1783,6 +1860,9 @@ populate_ingredients (GrEditPage *page,
         ingredients = gr_ingredients_list_new (text);
         segs = gr_ingredients_list_get_segments (ingredients);
         for (j = 0; segs[j]; j++) {
+                GtkWidget *list;
+                g_auto(GStrv) ings = NULL;
+
                 list = add_ingredients_segment (page, segs[j]);
                 ings = gr_ingredients_list_get_ingredients (ingredients, segs[j]);
                 for (i = 0; ings[i]; i++) {
@@ -1807,10 +1887,18 @@ populate_ingredients (GrEditPage *page,
         gtk_widget_set_margin_top (button, 20);
         gtk_box_pack_end (GTK_BOX (page->ingredients_box), button, FALSE, TRUE, 0);
         g_signal_connect (button, "clicked", G_CALLBACK (add_list), page);
-
         update_segments (page);
 }
 
+static void
+scroll_up (GrEditPage *page)
+{
+        GtkAdjustment *adj;
+
+        adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (page->main_content));
+        gtk_adjustment_set_value (adj, gtk_adjustment_get_lower (adj));
+}
+
 void
 gr_edit_page_clear (GrEditPage *page)
 {
@@ -1819,7 +1907,7 @@ gr_edit_page_clear (GrEditPage *page)
 
         gr_image_viewer_revert_changes (GR_IMAGE_VIEWER (page->images));
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         gtk_label_set_label (GTK_LABEL (page->name_label), _("Name Your Recipe"));
         gtk_entry_set_text (GTK_ENTRY (page->name_entry), "");
@@ -1853,6 +1941,8 @@ gr_edit_page_clear (GrEditPage *page)
 
         g_free (page->author);
         page->author = g_strdup (gr_recipe_store_get_user_key (store));
+
+        scroll_up (page);
 }
 
 static void
@@ -1971,7 +2061,7 @@ gr_edit_page_edit (GrEditPage *page,
 
         gr_image_viewer_revert_changes (GR_IMAGE_VIEWER (page->images));
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         name = gr_recipe_get_name (recipe);
         serves = gr_recipe_get_serves (recipe);
@@ -1990,6 +2080,7 @@ gr_edit_page_edit (GrEditPage *page,
 
         g_object_get (recipe, "images", &images, NULL);
 
+        g_free (page->author);
         page->author = g_strdup (author);
 
         chef = gr_recipe_store_get_chef (store, author ? author : "");
@@ -2033,6 +2124,8 @@ gr_edit_page_edit (GrEditPage *page,
         }
 
         update_default_image_button (page);
+
+        scroll_up (page);
 }
 
 gboolean
@@ -2055,7 +2148,7 @@ gr_edit_page_save (GrEditPage *page)
         GrDiets diets;
         g_autoptr(GArray) images = NULL;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         name = gtk_entry_get_text (GTK_ENTRY (page->name_entry));
         cuisine = get_combo_value (GTK_COMBO_BOX (page->cuisine_combo));
