@@ -30,7 +30,6 @@
 #include "gr-utils.h"
 #include "gr-ingredients-list.h"
 #include "gr-images.h"
-#include "gr-app.h"
 
 
 /**
@@ -39,30 +38,93 @@
  *
  * The current storage layer is very simple-minded. We just write data out
  * as a number of key files:
- * - recipes.db for recipes, with each recipe being its own group
+ * - recipes.db for recipes
  * - chefs.db for chefs, with each chef being its own group
+ * - picks.db, with lists of things to put on the landing page
  *
- * There's a few auxiliary files:
- * - picks, with lists of things to put on the landing page
- * - favorites.db, with a list of the user's favorites
- * - shopping.db, with the current shppping list
- * - user, containing the user id of the user
- * - cooking, with a counter for how often cooking mode was launched
- *
- * Some of these files are preinstalled (picks.db), some are only per-user
- * (favorites.db, shopping.db, user, cooking), and some are both, with the
- * per-user list overlaying the preinstalled one.
+ * Some of these files are preinstalled (picks.db), and some are both, with
+ * the per-user list overlaying the preinstalled one.
  *
  * Data from the preinstalled files is treated as readonly unless it
  * belongs to the current user.
  *
  * Data is always written to the per-user files.
  *
+ * The recipes.db format
+ * ---------------------
+ *
+ * Each recipe is its own group, with the recipe ID as the group name.
+ * Recipes have unique IDs of the form R_<dish_name>_by_<chef_ID>.
+ * Recognized keys:
+ *  Name (string)
+ *  Author (string, containing the chef ID)
+ *  Description (string)
+ *  Cuisine (string)
+ *  Season (string)
+ *  Category (string)
+ *  PrepTime (string)
+ *  CookTime (string)
+ *  Ingredients (string, with one ingredient per \n-separated line, each line having
+ *               amount, unit, ingredient and segment fields, separated by tab)
+ *  Instructions (string, with steps separated by \n\n, and optionally embedded markup
+ *                for timers, images or temperatures)
+ *  Notes (string)
+ *  Serves (integer)
+ *  Diets (integer)
+ *  Images (string list)
+ *  DefaultImage (integer)
+ *  Created (string, containing a timestamp)
+ *  Modified (string, containing a timestamp)
+ *
+ * The chefs.db format
+ * -------------------
+ *
+ * Each chef is its own group, with the chef ID as the group name.
+ * Recognized keys:
+ *  Name (string)
+ *  Fullname (string)
+ *  Description (string)
+ *  Image (string)
+ *
+ * The picks.db format
+ * -------------------
+ *
+ * This keyfile has a single group named Content. It contains the following
+ * keys:
+ *
+ *  Todays (string list, containing IDs of recipes to show in the top part
+ *          of the landing page)
+ *  Picks (string list, containing IDs of recipes to show in the Editors Picks
+ *         section of the landing page)
+ *  Chefs (string list, containing IDs of chefs to show in the Featured
+ *         GNOME Chefs part of the landing page)
+ *
  * Data at runtime
  * ---------------
  *
  * At runtime, we keep GrRecipe and GrChef objects in two separate hash tables.
- * Recipes have unique IDs of the form R_<dish_name>_by_<chef_id>.
+ *
+ * Ancillary data
+ * --------------
+ *
+ * Ancillary data is kept in gsettings:
+ *  - the user id of the user
+ *  - the counter for how often cooking mode was launched
+ *  - the list of favorites and the last-change timestamp for it
+ *  - the shopping list, with recipes, serving counts and removed ingredients
+ *
+ * Export/Import file format
+ * -------------------------
+ *
+ * We create a compressed archive (tar.gz), with the following files:
+ *
+ * recipes.db - same keyfile format as the main db
+ * chefs.db - same keyfile format as the main db
+ * images/ - contains any images that are referred to in the .db files
+ *
+ * The file name extension we use is .gnome-recipes-export, and we install
+ * am mime type 'application/vnd.gnome.recipes.export' for this extension
+ * and claim to be a mime handler for it.
  */
 
 struct _GrRecipeStore
@@ -75,8 +137,9 @@ struct _GrRecipeStore
         char **todays;
         char **picks;
         char **favorites;
-        char **shopping;
-        int   *shopping_serves;
+        char **export_list;
+        GVariantDict *shopping_list;
+        char **shopping_removed;
         char **featured_chefs;
         char *user;
 
@@ -99,7 +162,7 @@ gr_recipe_store_finalize (GObject *object)
         g_strfreev (self->todays);
         g_strfreev (self->picks);
         g_strfreev (self->favorites);
-        g_strfreev (self->shopping);
+        g_variant_dict_unref (self->shopping_list);
         g_strfreev (self->featured_chefs);
         g_free (self->user);
 
@@ -126,10 +189,10 @@ load_recipes (GrRecipeStore *self,
                 if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
                         g_error ("Failed to load recipe db: %s", error->message);
                 else
-                        g_message ("No recipe db at: %s", path);
+                        g_info ("No recipe db at: %s", path);
                 return FALSE;
         }
-        g_message ("Load recipe db: %s", path);
+        g_info ("Load recipe db: %s", path);
 
         groups = g_key_file_get_groups (keyfile, &length);
         for (i = 0; i < length; i++) {
@@ -429,7 +492,7 @@ save_recipes (GrRecipeStore *self)
                 dir = get_user_data_dir ();
         path = g_build_filename (dir, "recipes.db", NULL);
 
-        g_message ("Save recipe db: %s", path);
+        g_info ("Save recipe db: %s", path);
 
         keys = g_hash_table_get_keys (self->recipes);
         keys = g_list_sort (keys, (GCompareFunc)strcmp);
@@ -544,11 +607,11 @@ load_picks (GrRecipeStore *self,
                 if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
                         g_error ("Failed to load picks db: %s", error->message);
                 else
-                        g_message ("No picks db at: %s", path);
+                        g_info ("No picks db at: %s", path);
                 return FALSE;
         }
 
-        g_message ("Load picks db: %s", path);
+        g_info ("Load picks db: %s", path);
 
         self->picks = g_key_file_get_string_list (keyfile, "Content", "Picks", NULL, &error);
         if (error) {
@@ -577,168 +640,72 @@ load_picks (GrRecipeStore *self,
         return TRUE;
 }
 
-static gboolean
-load_favorites (GrRecipeStore *self,
-                const char    *dir)
+static void
+load_favorites (GrRecipeStore *self)
 {
-        g_autofree char *path = NULL;
-        g_autoptr(GKeyFile) keyfile = NULL;
-        g_autoptr(GError) error = NULL;
-        g_autofree char *tmp = NULL;
-        const char *key;
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        gint64 timestamp;
 
-        keyfile = g_key_file_new ();
-
-        path = g_build_filename (dir, "favorites.db", NULL);
-
-        if (!g_key_file_load_from_file (keyfile, path, G_KEY_FILE_NONE, &error)) {
-                if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-                        g_error ("Failed to load favorites db: %s", error->message);
-                else
-                        g_message ("No favorites db at: %s", path);
-                return FALSE;
-        }
-
-        g_message ("Load favorites db: %s", path);
-
-        if (g_key_file_has_key (keyfile, "Content", "Recipes", NULL))
-                key = "Recipes";
-        else
-                key = "Favorites";
-
-        self->favorites = g_key_file_get_string_list (keyfile, "Content", key, NULL, &error);
-        if (error) {
-                if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                        g_warning ("Failed to load favorites: %s", error->message);
-                }
-                g_clear_error (&error);
-        }
-
-        tmp = g_key_file_get_string (keyfile, "Content", "LastChange", &error);
-        if (error) {
-                if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                        g_warning ("Failed to load favorites: %s", error->message);
-                }
-                g_clear_error (&error);
-        }
-
-        if (tmp)
-                self->favorite_change = date_time_from_string (tmp);
-
-        return TRUE;
+        self->favorites = g_settings_get_strv (settings, "favorites");
+        timestamp = g_settings_get_int64 (settings, "favorites-last-change");
+        self->favorite_change = g_date_time_new_from_unix_utc (timestamp);
 }
 
 static void
 save_favorites (GrRecipeStore *self)
 {
-        g_autoptr(GKeyFile) keyfile = NULL;
-        g_autofree char *path = NULL;
-        g_autoptr(GError) error = NULL;
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        gint64 timestamp;
 
-        keyfile = g_key_file_new ();
-
-        path = g_build_filename (get_user_data_dir (), "favorites.db", NULL);
-
-        g_message ("Save favorites db: %s", path);
-
-        g_key_file_set_string_list (keyfile, "Content", "Recipes", (const char * const *)self->favorites, g_strv_length (self->favorites));
-
-        if (self->favorite_change) {
-                g_autofree char *tmp = NULL;
-
-                tmp = date_time_to_string (self->favorite_change);
-                g_key_file_set_string (keyfile, "Content", "LastChange", tmp);
-        }
-
-        if (!g_key_file_save_to_file (keyfile, path, &error)) {
-                g_error ("Failed to save recipe database: %s", error->message);
-        }
+        g_settings_set_strv (settings, "favorites", (const char * const *)self->favorites);
+        timestamp = g_date_time_to_unix (self->favorite_change);
+        g_settings_set_int64 (settings, "favorites-last-change", timestamp);
 }
 
-static gboolean
-load_shopping (GrRecipeStore *self,
-               const char    *dir)
+static void
+load_export_list (GrRecipeStore *self)
 {
-        g_autofree char *path = NULL;
-        g_autoptr(GKeyFile) keyfile = NULL;
-        g_autoptr(GError) error = NULL;
-        g_autofree char *tmp = NULL;
-        gsize len1, len2;
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
 
-        keyfile = g_key_file_new ();
+        self->export_list = g_settings_get_strv (settings, "export-list");
+}
 
-        path = g_build_filename (dir, "shopping.db", NULL);
+static void
+save_export_list (GrRecipeStore *self)
+{
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
 
-        if (!g_key_file_load_from_file (keyfile, path, G_KEY_FILE_NONE, &error)) {
-                if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-                        g_error ("Failed to load shopping db: %s", error->message);
-                else
-                        g_message ("No shopping db at: %s", path);
-                return FALSE;
-        }
+        g_settings_set_strv (settings, "export-list", (const char * const *)self->export_list);
+}
 
-        g_message ("Load shopping db: %s", path);
+static void
+load_shopping (GrRecipeStore *self)
+{
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        g_autoptr(GVariant) value = NULL;
+        gint64 timestamp;
 
-        self->shopping = g_key_file_get_string_list (keyfile, "Content", "Recipes", &len1, &error);
-        if (error) {
-                if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                        g_warning ("Failed to load shopping: %s", error->message);
-                }
-                g_clear_error (&error);
-        }
-        self->shopping_serves = g_key_file_get_integer_list (keyfile, "Content", "Serves", &len2, &error);
-        if (error) {
-                if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                        g_warning ("Failed to load shopping: %s", error->message);
-                }
-                g_clear_error (&error);
-        }
-
-        if (len1 != len2) {
-                g_warning ("Failed to load shopping: Recipes and Serves lists have different lengths");
-        }
-
-        tmp = g_key_file_get_string (keyfile, "Content", "LastChange", &error);
-        if (error) {
-                if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-                        g_warning ("Failed to load shopping: %s", error->message);
-                }
-                g_clear_error (&error);
-        }
-
-        if (tmp)
-                self->shopping_change = date_time_from_string (tmp);
-
-        return TRUE;
+        value = g_settings_get_value (settings, "shopping-list");
+        self->shopping_list = g_variant_dict_new (value);
+        timestamp = g_settings_get_int64 (settings, "shopping-list-last-change");
+        self->shopping_change = g_date_time_new_from_unix_utc (timestamp);
+        self->shopping_removed = g_settings_get_strv (settings, "shopping-list-removed-ingredients");
 }
 
 static void
 save_shopping (GrRecipeStore *self)
 {
-        g_autoptr(GKeyFile) keyfile = NULL;
-        g_autofree char *path = NULL;
-        g_autoptr(GError) error = NULL;
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        g_autoptr(GVariant) value = NULL;
+        gint64 timestamp;
 
-        keyfile = g_key_file_new ();
-
-        path = g_build_filename (get_user_data_dir (), "shopping.db", NULL);
-
-        g_message ("Save shopping db: %s", path);
-
-        g_key_file_set_string_list (keyfile, "Content", "Recipes", (const char * const *)self->shopping, g_strv_length (self->shopping));
-
-        g_key_file_set_integer_list (keyfile, "Content", "Serves", self->shopping_serves, g_strv_length (self->shopping));
-
-        if (self->shopping_change) {
-                g_autofree char *tmp = NULL;
-
-                tmp = date_time_to_string (self->shopping_change);
-                g_key_file_set_string (keyfile, "Content", "LastChange", tmp);
-        }
-
-        if (!g_key_file_save_to_file (keyfile, path, &error)) {
-                g_error ("Failed to save shopping database: %s", error->message);
-        }
+        g_settings_set_strv (settings, "shopping-list-removed-ingredients", (const char * const *)self->shopping_removed);
+        value = g_variant_ref_sink (g_variant_dict_end (self->shopping_list));
+        g_settings_set_value (settings, "shopping-list", value);
+        g_variant_dict_unref (self->shopping_list);
+        self->shopping_list = g_variant_dict_new (value);
+        timestamp = g_date_time_to_unix (self->shopping_change);
+        g_settings_set_int64 (settings, "shopping-list-last-change", timestamp);
 }
 
 static gboolean
@@ -766,11 +733,11 @@ load_chefs (GrRecipeStore *self,
                 if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
                         g_error ("Failed to load chefs db: %s", error->message);
                 else
-                        g_message ("No chefs db at: %s", path);
+                        g_info ("No chefs db at: %s", path);
                 return FALSE;
         }
 
-        g_message ("Load chefs db: %s", path);
+        g_info ("Load chefs db: %s", path);
 
         groups = g_key_file_get_groups (keyfile, &length);
         for (i = 0; i < length; i++) {
@@ -860,7 +827,7 @@ save_chefs (GrRecipeStore *store)
                 dir = get_user_data_dir ();
         path = g_build_filename (dir, "chefs.db", NULL);
 
-        g_message ("Save chefs db: %s", path);
+        g_info ("Save chefs db: %s", path);
 
         keys = g_hash_table_get_keys (store->chefs);
         keys = g_list_sort (keys, (GCompareFunc)strcmp);
@@ -904,51 +871,23 @@ save_chefs (GrRecipeStore *store)
 static void
 save_user (GrRecipeStore *self)
 {
-        g_autofree char *path = NULL;
-
-        path = g_build_filename (get_user_data_dir (), "user", NULL);
-
-        g_message ("Save user id: %s", path);
-
-        if (self->user == NULL || self->user[0] == '\0') {
-                g_unlink (path);
-        }
-        else {
-                g_autoptr(GError) error = NULL;
-
-                if (!g_file_set_contents (path, self->user, -1, &error)) {
-                        g_error ("Failed to save user id: %s", error->message);
-                }
-        }
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        g_settings_set_string (settings, "user", self->user ? self->user : "");
 }
 
-static gboolean
+static void
 load_user (GrRecipeStore *self,
            const char    *dir)
 {
-        g_autofree char *path = NULL;
-        g_autofree char *contents = NULL;
-        g_autoptr(GError) error = NULL;
+        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        g_autofree char *user = g_settings_get_string (settings, "user");
 
-        path = g_build_filename (dir, "user", NULL);
-        if (!g_file_get_contents (path, &contents, NULL, &error)) {
-                if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-                        g_error ("Failed to load user id: %s", error->message);
-                else {
-                        self->user = g_strdup (g_get_user_name ());
-                        save_user (self);
-                }
-                return FALSE;
+        if (user[0] == '\0') {
+                self->user = g_strdup (g_get_user_name ());
+                save_user (self);
         }
 
-        g_message ("Load user id: %s", path);
-
-        self->user = g_strdup (contents);
-
-        if (self->user[strlen (self->user) - 1] == '\n')
-                self->user[strlen (self->user) - 1] = '\0';
-
-        return TRUE;
+        self->user = g_strdup (user);
 }
 
 static void
@@ -956,7 +895,6 @@ gr_recipe_store_init (GrRecipeStore *self)
 {
         const char *data_dir;
         const char *user_dir;
-        const char *old_dir;
         g_autofree char *current_dir = NULL;
         g_autofree char *uninstalled_dir = NULL;
 
@@ -973,14 +911,14 @@ gr_recipe_store_init (GrRecipeStore *self)
         if (g_getenv ("RECIPES_DATA_DIR")) {
                 const char *dir = g_getenv ("RECIPES_DATA_DIR");
 
-                g_message ("Loading data from RECIPES_DATA_DIR: %s", dir);
+                g_info ("Loading data from RECIPES_DATA_DIR: %s", dir);
 
                 load_recipes (self, dir, FALSE);
                 load_chefs (self, dir, FALSE);
                 load_picks (self, dir);
 
-                g_message ("%d recipes loaded", g_hash_table_size (self->recipes));
-                g_message ("%d chefs loaded", g_hash_table_size (self->chefs));
+                g_info ("%d recipes loaded", g_hash_table_size (self->recipes));
+                g_info ("%d chefs loaded", g_hash_table_size (self->chefs));
 
                 return;
         }
@@ -995,35 +933,13 @@ gr_recipe_store_init (GrRecipeStore *self)
 
         /* Now load saved data */
         load_recipes (self, user_dir, FALSE);
-        load_favorites (self, user_dir);
-        load_shopping (self, user_dir);
+        load_favorites (self);
+        load_export_list (self);
+        load_shopping (self);
         load_chefs (self, user_dir, FALSE);
 
-        /* Look for data from before the recipes -> gnome-recipes rename */
-        old_dir = get_old_user_data_dir ();
-        if (load_recipes (self, old_dir, FALSE)) {
-                g_autofree char *filename = g_build_filename (old_dir, "recipes.db", NULL);
-                g_remove (filename);
-                save_recipes (self);
-        }
-        if (load_favorites (self, old_dir)) {
-                g_autofree char *filename = g_build_filename (old_dir, "favorites.db", NULL);
-                g_remove (filename);
-                save_favorites (self);
-        }
-        if (load_shopping (self, old_dir)) {
-                g_autofree char *filename = g_build_filename (old_dir, "shopping.db", NULL);
-                g_remove (filename);
-                save_shopping (self);
-        }
-        if (load_chefs (self, old_dir, FALSE)) {
-                g_autofree char *filename = g_build_filename (old_dir, "chefs.db", NULL);
-                g_remove (filename);
-                save_chefs (self);
-        }
-
-        g_message ("%d recipes loaded", g_hash_table_size (self->recipes));
-        g_message ("%d chefs loaded", g_hash_table_size (self->chefs));
+        g_info ("%d recipes loaded", g_hash_table_size (self->recipes));
+        g_info ("%d chefs loaded", g_hash_table_size (self->chefs));
 }
 
 static guint add_signal;
@@ -1238,6 +1154,29 @@ gr_recipe_store_get_chef_keys (GrRecipeStore *self,
         return (char **)g_hash_table_get_keys_as_array (self->chefs, length);
 }
 
+char **
+gr_recipe_store_get_contributors (GrRecipeStore *self,
+                                  guint         *length)
+{
+        GHashTableIter iter;
+        GrRecipe *recipe;
+        g_autoptr(GHashTable) chefs = NULL;
+
+        chefs = g_hash_table_new (g_str_hash, g_str_equal);
+
+        g_hash_table_iter_init (&iter, self->recipes);
+        while (g_hash_table_iter_next (&iter, NULL, (gpointer *)&recipe)) {
+                if (gr_recipe_is_contributed (recipe)) {
+                        GrChef *chef;
+
+                        chef = g_hash_table_lookup (self->chefs, gr_recipe_get_author (recipe));
+                        g_hash_table_add (chefs, (gpointer)gr_chef_get_fullname (chef));
+                }
+        }
+
+        return (char **)g_hash_table_get_keys_as_array (chefs, length);
+}
+
 const char *
 gr_recipe_store_get_user_key (GrRecipeStore *self)
 {
@@ -1396,13 +1335,53 @@ gr_recipe_store_get_all_ingredients (GrRecipeStore *self,
         return result;
 }
 
-void
-gr_recipe_store_add_favorite (GrRecipeStore *self,
-                              GrRecipe      *recipe)
+static void
+strv_prepend (char       ***strv_in,
+              const char   *s)
 {
         char **strv;
         int length;
         int i;
+
+        length = g_strv_length (*strv_in);
+        strv = g_new (char *, length + 2);
+        strv[0] = g_strdup (s);
+        for (i = 0; i < length; i++)
+                strv[i + 1] = (*strv_in)[i];
+        strv[length + 1] = NULL;
+
+        g_free (*strv_in);
+        *strv_in = strv;
+}
+
+static void
+strv_remove (char       ***strv_in,
+             const char   *s)
+{
+        int i, j;
+        char **strv;
+        int length;
+
+        length = g_strv_length (*strv_in);
+        strv = g_new (char *, length + 1);
+
+        for (i = 0, j = 0; i < length; i++) {
+                if (strcmp (strv[i], s) == 0) {
+                        g_free (strv[i]);
+                        continue;
+                }
+                strv[j++] = strv[i];
+        }
+        strv[j] = NULL;
+
+        g_free (*strv_in);
+        *strv_in = strv;
+}
+
+void
+gr_recipe_store_add_favorite (GrRecipeStore *self,
+                              GrRecipe      *recipe)
+{
         const char *id;
 
         id = gr_recipe_get_id (recipe);
@@ -1410,15 +1389,7 @@ gr_recipe_store_add_favorite (GrRecipeStore *self,
         if (g_strv_contains ((const char * const*)self->favorites, id))
                 return;
 
-        length = g_strv_length (self->favorites);
-        strv = g_new (char *, length + 2);
-        strv[0] = g_strdup (id);
-        for (i = 0; i < length; i++)
-                strv[i + 1] = self->favorites[i];
-        strv[length + 1] = NULL;
-
-        g_free (self->favorites);
-        self->favorites = strv;
+        strv_prepend (&self->favorites, id);
 
         if (self->favorite_change)
                 g_date_time_unref (self->favorite_change);
@@ -1477,11 +1448,57 @@ gr_recipe_store_last_favorite_change (GrRecipeStore *self)
         return self->favorite_change;
 }
 
-static void
-ensure_shopping (GrRecipeStore *self)
+void
+gr_recipe_store_add_export (GrRecipeStore *self,
+                            GrRecipe      *recipe)
 {
-        if (!self->shopping)
-                self->shopping = g_new0 (char *, 1);
+        const char *id;
+
+        id = gr_recipe_get_id (recipe);
+
+        if (g_strv_contains ((const char * const*)self->export_list, id))
+                return;
+
+        strv_prepend (&self->export_list, id);
+
+        save_export_list (self);
+}
+
+void
+gr_recipe_store_remove_export (GrRecipeStore *self,
+                               GrRecipe      *recipe)
+{
+        int i, j;
+        const char *id;
+
+        id = gr_recipe_get_id (recipe);
+
+        for (i = 0; self->export_list[i]; i++) {
+                if (strcmp (self->export_list[i], id) == 0) {
+                        g_free (self->export_list[i]);
+                        for (j = i; self->export_list[j]; j++) {
+                                self->export_list[j] = self->export_list[j + 1];
+                        }
+                        break;
+                }
+        }
+
+        save_export_list (self);
+}
+
+const char **
+gr_recipe_store_get_export_list (GrRecipeStore *self)
+{
+        return (const char **)self->export_list;
+}
+
+void
+gr_recipe_store_clear_export_list (GrRecipeStore *self)
+{
+        g_strfreev (self->export_list);
+        self->export_list = g_new0 (char *, 1);
+
+        save_export_list (self);
 }
 
 void
@@ -1489,41 +1506,11 @@ gr_recipe_store_add_to_shopping (GrRecipeStore *self,
                                  GrRecipe      *recipe,
                                  int            serves)
 {
-        char **strv;
-        int *intv;
-        int length;
-        int i;
         const char *id;
 
-        ensure_shopping (self);
-
         id = gr_recipe_get_id (recipe);
+        g_variant_dict_insert (self->shopping_list, id, "u", (guint)serves);
 
-        for (i = 0; self->shopping[i]; i++) {
-                if (strcmp (self->shopping[i], id) == 0) {
-                        self->shopping_serves[i] = serves;
-                        goto save;
-                }
-        }
-
-        length = g_strv_length (self->shopping);
-        strv = g_new (char *, length + 2);
-        strv[0] = g_strdup (id);
-        intv = g_new (int, length + 2);
-        intv[0] = serves;
-        for (i = 0; i < length; i++) {
-                strv[i + 1] = self->shopping[i];
-                intv[i + 1] = self->shopping_serves[i];
-        }
-        strv[length + 1] = NULL;
-        intv[length + 1] = 0;
-
-        g_free (self->shopping);
-        g_free (self->shopping_serves);
-        self->shopping = strv;
-        self->shopping_serves = intv;
-
-save:
         if (self->shopping_change)
                 g_date_time_unref (self->shopping_change);
         self->shopping_change = g_date_time_new_now_utc ();
@@ -1537,23 +1524,10 @@ void
 gr_recipe_store_remove_from_shopping (GrRecipeStore *self,
                                       GrRecipe      *recipe)
 {
-        int i, j;
         const char *id;
 
-        ensure_shopping (self);
-
         id = gr_recipe_get_id (recipe);
-
-        for (i = 0; self->shopping[i]; i++) {
-                if (strcmp (self->shopping[i], id) == 0) {
-                        g_free (self->shopping[i]);
-                        for (j = i; self->shopping[j]; j++) {
-                                self->shopping[j] = self->shopping[j + 1];
-                                self->shopping_serves[j] = self->shopping_serves[j + 1];
-                        }
-                        break;
-                }
-        }
+        g_variant_dict_remove (self->shopping_list, id);
 
         if (self->shopping_change)
                 g_date_time_unref (self->shopping_change);
@@ -1564,18 +1538,33 @@ gr_recipe_store_remove_from_shopping (GrRecipeStore *self,
         g_signal_emit (self, changed_signal, 0, recipe);
 }
 
+void
+gr_recipe_store_clear_shopping_list (GrRecipeStore *self)
+{
+        const char **empty[1] = { NULL };
+
+        g_variant_dict_unref (self->shopping_list);
+        self->shopping_list = g_variant_dict_new (NULL);
+
+        g_strfreev (self->shopping_removed);
+        self->shopping_removed = g_strdupv ((char **)empty);
+
+        if (self->shopping_change)
+                g_date_time_unref (self->shopping_change);
+        self->shopping_change = g_date_time_new_now_utc ();
+
+        save_shopping (self);
+}
+
 gboolean
 gr_recipe_store_is_in_shopping (GrRecipeStore *self,
                                 GrRecipe      *recipe)
 {
         const char *id;
 
-        if (self->shopping == NULL)
-                return FALSE;
-
         id = gr_recipe_get_id (recipe);
 
-        return g_strv_contains ((const char *const*)self->shopping, id);
+        return g_variant_dict_contains (self->shopping_list, id);
 }
 
 int
@@ -1583,19 +1572,48 @@ gr_recipe_store_get_shopping_serves (GrRecipeStore *self,
                                      GrRecipe      *recipe)
 {
         const char *id;
-        int i;
+        guint serves;
 
         id = gr_recipe_get_id (recipe);
-
-        for (i = 0; self->shopping[i]; i++) {
-                if (strcmp (self->shopping[i], id) == 0) {
-                        return self->shopping_serves[i];
-                }
-        }
+        if (g_variant_dict_lookup (self->shopping_list, id, "u", &serves))
+                return (int)serves;
 
         return 0;
 }
 
+gboolean
+gr_recipe_store_not_shopping_ingredient (GrRecipeStore *self,
+                                         const char    *ingredient)
+{
+        return g_strv_contains ((const char * const *)self->shopping_removed, ingredient);
+}
+
+void
+gr_recipe_store_remove_shopping_ingredient (GrRecipeStore *self,
+                                            const char    *ingredient)
+{
+        strv_prepend (&self->shopping_removed, ingredient);
+
+        if (self->shopping_change)
+                g_date_time_unref (self->shopping_change);
+        self->shopping_change = g_date_time_new_now_utc ();
+
+        save_shopping (self);
+}
+
+void
+gr_recipe_store_readd_shopping_ingredient (GrRecipeStore *self,
+                                           const char    *ingredient)
+{
+        strv_remove (&self->shopping_removed, ingredient);
+
+        if (self->shopping_change)
+                g_date_time_unref (self->shopping_change);
+        self->shopping_change = g_date_time_new_now_utc ();
+
+        save_shopping (self);
+}
+
 GDateTime *
 gr_recipe_store_last_shopping_change (GrRecipeStore *self)
 {
@@ -1691,7 +1709,7 @@ gr_recipe_search_new (void)
 
         search = GR_RECIPE_SEARCH (g_object_new (GR_TYPE_RECIPE_SEARCH, NULL));
 
-        search->store = g_object_ref (gr_app_get_recipe_store (GR_APP (g_application_get_default ())));
+        search->store = g_object_ref (gr_recipe_store_get ());
 
         return search;
 }
@@ -2022,3 +2040,14 @@ static void
 gr_recipe_search_init (GrRecipeSearch *self)
 {
 }
+
+GrRecipeStore *
+gr_recipe_store_get (void)
+{
+        static GrRecipeStore *store;
+
+        if (store == NULL)
+                store = gr_recipe_store_new ();
+
+        return store;
+}
