@@ -27,7 +27,6 @@
 #include "gr-recipe-store.h"
 #include "gr-recipe.h"
 #include "gr-recipe-small-tile.h"
-#include "gr-app.h"
 #include "gr-utils.h"
 #include "gr-ingredients-list.h"
 #include "gr-window.h"
@@ -111,15 +110,15 @@ recount_recipes (GrShoppingPage *page)
         int count;
         g_autofree char *tmp = NULL;
         GtkWidget *window;
-        
+
         children = gtk_container_get_children (GTK_CONTAINER (page->recipe_list));
         count = g_list_length (children);
         g_list_free (children);
-        
+
         g_free (page->title);
         page->title = g_strdup_printf (ngettext ("Buy Ingredients (%d recipe)",
-                                                 "Buy Ingredients (%d recipes)", page->recipe_count),
-                                       page->recipe_count);
+                                                 "Buy Ingredients (%d recipes)", count),
+                                       count);
         g_object_notify (G_OBJECT (page), "title");
 
         tmp = g_strdup_printf (ngettext ("%d Recipe marked for preparation",
@@ -127,6 +126,11 @@ recount_recipes (GrShoppingPage *page)
                                count);
         gtk_label_set_label (GTK_LABEL (page->recipe_count_label), tmp);
         if (count == 0) {
+                GrRecipeStore *store;
+
+                store = gr_recipe_store_get ();
+                gr_recipe_store_clear_shopping_list (store);
+
                 window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
                 gr_window_go_back (GR_WINDOW (window));
         }
@@ -155,12 +159,17 @@ static Ingredient *
 ingredient_new (const char *ingredient)
 {
         Ingredient *ing;
+        GrRecipeStore *store;
+
+        store = gr_recipe_store_get ();
 
         ing = g_new0 (Ingredient, 1);
         ing->ingredient = g_strdup (ingredient);
         ing->units = g_array_new (FALSE, TRUE, sizeof (Unit));
         g_array_set_clear_func (ing->units, clear_unit);
 
+        ing->removed = gr_recipe_store_not_shopping_ingredient (store, ingredient);
+
         return ing;
 }
 
@@ -267,6 +276,9 @@ remove_ingredient (GtkButton *button, GrShoppingPage *page)
         const char *name;
         const char *unit;
         Ingredient *ing;
+        GrRecipeStore *store;
+
+        store = gr_recipe_store_get ();
 
         row = gtk_widget_get_ancestor (GTK_WIDGET (button), GTK_TYPE_LIST_BOX_ROW);
 
@@ -278,6 +290,8 @@ remove_ingredient (GtkButton *button, GrShoppingPage *page)
         ing = (Ingredient *)g_hash_table_lookup (page->ingredients, name);
         ing->removed = TRUE;
 
+        gr_recipe_store_remove_shopping_ingredient (store, name);
+
         add_removed_row (page, unit, name);
 
         gtk_widget_destroy (row);
@@ -387,6 +401,9 @@ removed_row_activated (GtkListBox     *list,
         const char *name;
         const char *unit;
         Ingredient *ing;
+        GrRecipeStore *store;
+
+        store = gr_recipe_store_get ();
 
         popover = gtk_widget_get_ancestor (GTK_WIDGET (row), GTK_TYPE_POPOVER);
         gtk_popover_popdown (GTK_POPOVER (popover));
@@ -399,6 +416,8 @@ removed_row_activated (GtkListBox     *list,
         ing = (Ingredient *)g_hash_table_lookup (page->ingredients, name);
         ing->removed = FALSE;
 
+        gr_recipe_store_readd_shopping_ingredient (store, name);
+
         add_ingredient_row (page, unit, name);
 
         gtk_widget_destroy (GTK_WIDGET (row));
@@ -504,7 +523,7 @@ serves_changed (GObject *object, GParamSpec *pspec, GrShoppingPage *page)
         recipe = gr_recipe_small_tile_get_recipe (tile);
         serves = gr_recipe_small_tile_get_serves (tile);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         gr_recipe_store_add_to_shopping (store, recipe, serves);
 }
@@ -525,7 +544,7 @@ search_hits_added (GrRecipeSearch *search,
         GrRecipeStore *store;
         GList *l;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         for (l = hits; l; l = l->next) {
                 GrRecipe *recipe = l->data;
@@ -574,31 +593,16 @@ search_finished (GrRecipeSearch *search,
 static void
 clear_list (GrShoppingPage *page)
 {
-        GList *children, *l;
         GrRecipeStore *store;
         GtkWidget *window;
-        GList *recipes;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
-
-        children = gtk_container_get_children (GTK_CONTAINER (page->recipe_list));
-        recipes = NULL;
-        for (l = children; l; l = l->next) {
-                GtkWidget *tile = gtk_bin_get_child (GTK_BIN (l->data));
-                GrRecipe *recipe = gr_recipe_small_tile_get_recipe (GR_RECIPE_SMALL_TILE (tile));
-                recipes = g_list_prepend (recipes, g_object_ref (recipe));
-        }
-        g_list_free (children);
+        store = gr_recipe_store_get ();
 
         container_remove_all (GTK_CONTAINER (page->ingredients_list));
         container_remove_all (GTK_CONTAINER (page->removed_list));
         container_remove_all (GTK_CONTAINER (page->recipe_list));
 
-        for (l = recipes; l; l = l->next) {
-                GrRecipe *recipe = l->data;
-                gr_recipe_store_remove_from_shopping (store, recipe);
-        }
-        g_list_free_full (recipes, g_object_unref);
+        gr_recipe_store_clear_shopping_list (store);
 
         g_hash_table_remove_all (page->ingredients);
 
@@ -717,7 +721,7 @@ mail_done (GObject      *source,
                 GObject *file_chooser;
                 GtkWidget *window;
 
-                g_message ("Sending mail failed: %s", error->message);
+                g_info ("Sending mail failed: %s", error->message);
 
                 window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
                 file_chooser = (GObject *)gtk_file_chooser_native_new (_("Save the shopping list"),
@@ -864,6 +868,12 @@ gr_shopping_page_populate (GrShoppingPage *self)
         gr_recipe_search_set_query (self->search, "is:shopping");
 }
 
+GList *
+gr_shopping_page_get_recipes (GrShoppingPage *self)
+{
+        return get_recipes (self);
+}
+
 static void
 recipe_removed (GrShoppingPage *page,
                 GrRecipe       *recipe)
@@ -898,7 +908,7 @@ recipe_added (GrShoppingPage *page,
         if (!gtk_widget_is_drawable (GTK_WIDGET (page)))
                 return;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         serves = gr_recipe_store_get_shopping_serves (store, recipe);
 
@@ -935,7 +945,7 @@ recipe_changed (GrShoppingPage *page,
         if (!gtk_widget_is_drawable (GTK_WIDGET (page)))
                 return;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         if (gr_recipe_store_is_in_shopping (store, recipe))
                 recipe_added (page, recipe);
@@ -948,7 +958,7 @@ connect_store_signals (GrShoppingPage *page)
 {
         GrRecipeStore *store;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         g_signal_connect_swapped (store, "recipe-removed", G_CALLBACK (recipe_removed), page);
         g_signal_connect_swapped (store, "recipe-changed", G_CALLBACK (recipe_changed), page);
