@@ -39,6 +39,7 @@
 #include "gr-cuisines-page.h"
 #include "gr-image-page.h"
 #include "gr-query-editor.h"
+#include "gr-recipe-store.h"
 #include "gr-recipe-importer.h"
 #include "gr-recipe-exporter.h"
 #include "gr-about-dialog.h"
@@ -58,12 +59,14 @@ struct _GrWindow
         GtkWidget *back_button;
         GtkWidget *search_button;
         GtkWidget *cooking_button;
+        GtkWidget *save_button;
         GtkWidget *search_bar;
         GtkWidget *main_stack;
         GtkWidget *recipes_page;
         GtkWidget *details_page;
         GtkWidget *edit_page;
         GtkWidget *list_page;
+        GtkWidget *transient_list_page;
         GtkWidget *chef_page;
         GtkWidget *shopping_page;
         GtkWidget *search_page;
@@ -81,6 +84,8 @@ struct _GrWindow
         guint remind_timeout_id;
         GtkWidget *shopping_added_revealer;
         guint shopping_timeout_id;
+        GtkWidget *shopping_done_revealer;
+        guint shopping_done_timeout_id;
 
         GObject *file_chooser;
         GrRecipeImporter *importer;
@@ -88,6 +93,7 @@ struct _GrWindow
 
         GtkWidget *chef_dialog;
         GtkWidget *about_dialog;
+        GtkWidget *news_dialog;
 
         GList *dialogs;
 
@@ -97,6 +103,34 @@ struct _GrWindow
 
 G_DEFINE_TYPE (GrWindow, gr_window, GTK_TYPE_APPLICATION_WINDOW)
 
+/* Back button theory:
+ *
+ * We maintain a stack of entries so we can go back multiple steps if
+ * the need arises. For each step, we need to record enough information
+ * to properly repopulate the page, including the headerbar and the
+ * state of the search bar.
+ *
+ * We don't want a full browser-style history here. Instead, navigation
+ * works somewhat stratified:
+ *
+ * - The 'main views': recipes, cuisines, search. You move between these
+ *   without any back information getting recorded.
+ *
+ * - Hierarchical navigation for categories, chefs and similar
+ *   'list-of-recipes' pages, as well as for the cuisine view. If you go
+ *   from a main view to a list page to an individual recipe, you can
+ *   retrace your steps with the back button.
+ *
+ * - Some lists are 'transient': they don't get added to the stack.
+ *   Examples are 'Imported recipes' and 'Ready to cook'. If you go from
+ *   these to an individual recipe and back, you end up on a main view
+ *
+ * - Individual recipe pages: details, edit and new. If you save a recipe,
+ *   we move you to the details page. Apart from that, these pages are
+ *   not participating in the back stack. If you navigate from a details
+ *   page to the chef page, the back button will bring you to the main view,
+ *   not back to the recipe you came from.
+ */
 typedef struct
 {
         char *page;
@@ -128,6 +162,9 @@ save_back_entry (GrWindow *window)
         page = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
         if (strcmp (page, "details") == 0 ||
             strcmp (page, "cooking") == 0 ||
+            strcmp (page, "image") == 0 ||
+            strcmp (page, "transient") == 0 ||
+            strcmp (page, "shopping") == 0 ||
             strcmp (page, "edit") == 0)
                 return;
 
@@ -182,6 +219,7 @@ new_recipe (GrWindow *window)
         save_back_entry (window);
 
         gr_edit_page_clear (GR_EDIT_PAGE (window->edit_page));
+        gtk_widget_grab_focus (window->edit_page);
 
         gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Add a New Recipe"));
 
@@ -190,6 +228,7 @@ new_recipe (GrWindow *window)
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "edit");
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "edit");
+        gtk_widget_set_sensitive (window->save_button,FALSE);
 }
 
 static void
@@ -267,9 +306,37 @@ static void
 visible_page_changed (GrWindow *window)
 {
         const char *visible;
+        gboolean searchable;
+        gboolean fullscreen;
+        GAction *action;
+        const char *actions[] = {
+                "chef-information",
+                "about",
+                "news",
+                "import",
+                "export",
+                "details",
+                "quit",
+                "report-issue"
+        };
+        int i;
 
         visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
 
+        searchable = strcmp (visible, "recipes") == 0 ||
+                     strcmp (visible, "cuisines") == 0 ||
+                     strcmp (visible, "search") == 0;
+        fullscreen = strcmp (visible, "image") == 0 ||
+                     strcmp (visible, "cooking") == 0;
+
+        action = g_action_map_lookup_action (G_ACTION_MAP (g_application_get_default ()), "search");
+        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), searchable);
+
+        for (i = 0; i < G_N_ELEMENTS (actions); i++) {
+                action = g_action_map_lookup_action (G_ACTION_MAP (g_application_get_default ()), actions[i]);
+                g_simple_action_set_enabled (G_SIMPLE_ACTION (action), !fullscreen);
+        }
+
         if (strcmp (visible, "search") != 0) {
                 g_signal_handlers_block_by_func (window->search_bar, search_changed, window);
                 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (window->search_button), FALSE);
@@ -343,47 +410,6 @@ gr_window_timer_expired (GrWindow *window,
         gr_cooking_page_timer_expired (GR_COOKING_PAGE (window->cooking_page), step);
 }
 
-static gboolean
-window_keypress_handler (GtkWidget *widget,
-                         GdkEvent  *event,
-                         gpointer   data)
-{
-        GrWindow *window = GR_WINDOW (widget);
-        GdkEventKey *e = (GdkEventKey *) event;
-        const char *visible;
-
-        visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
-
-        if (strcmp (visible, "image") == 0) {
-                if (e->keyval == GDK_KEY_Escape) {
-                        gr_window_show_image (window, NULL, -1);
-                        return GDK_EVENT_STOP;
-                }
-        }
-
-        if (strcmp (visible, "cooking") == 0)
-                return gr_cooking_page_handle_event (GR_COOKING_PAGE (window->cooking_page), event);
-
-        if (strcmp (visible, "recipes") != 0 &&
-            strcmp (visible, "cuisines") != 0 &&
-            strcmp (visible, "search") != 0)
-                return GDK_EVENT_PROPAGATE;
-
-        /* handle ctrl+f shortcut */
-        if (event->type == GDK_KEY_PRESS) {
-                if ((e->state & GDK_CONTROL_MASK) > 0 && e->keyval == GDK_KEY_f) {
-                        if (!gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (window->search_bar))) {
-                                gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), TRUE);
-                        } else {
-                                gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), FALSE);
-                        }
-                        return GDK_EVENT_PROPAGATE;
-                }
-        }
-
-        return gr_query_editor_handle_event (GR_QUERY_EDITOR (window->search_bar), event);
-}
-
 static gboolean
 window_keypress_handler_before (GtkWidget *widget,
                                 GdkEvent  *event,
@@ -400,6 +426,24 @@ window_keypress_handler_before (GtkWidget *widget,
         return GDK_EVENT_PROPAGATE;
 }
 
+static gboolean
+window_keypress_handler_after (GtkWidget *widget,
+                               GdkEvent  *event,
+                               gpointer   data)
+{
+        GrWindow *window = GR_WINDOW (widget);
+        const char *visible;
+
+        visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
+
+        if (strcmp (visible, "recipes") == 0 ||
+            strcmp (visible, "cuisines") == 0 ||
+            strcmp (visible, "search") == 0)
+                return gr_query_editor_handle_event (GR_QUERY_EDITOR (window->search_bar), event);
+
+        return GDK_EVENT_PROPAGATE;
+}
+
 static gboolean
 window_buttonpress_handler (GtkWidget *widget,
                             GdkEvent  *event,
@@ -434,6 +478,55 @@ window_mapped_handler (GtkWidget *widget)
         gr_cuisines_page_unexpand (GR_CUISINES_PAGE (window->cuisines_page));
 }
 
+static void
+response_cb (GtkDialog *dialog,
+             int        response,
+             gpointer   data)
+{
+        GrWindow *window = data;
+
+        if (response == GTK_RESPONSE_YES)
+                gtk_widget_destroy (GTK_WIDGET (window));
+        else
+                gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static gboolean
+window_delete_handler (GtkWidget *widget)
+{
+        GrWindow *window = GR_WINDOW (widget);
+        const char *visible;
+        gboolean unsaved;
+
+        visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
+        g_object_get (window->edit_page, "unsaved", &unsaved, NULL);
+
+        if (strcmp (visible, "edit") == 0 && unsaved) {
+                GtkWidget *dialog;
+                GrRecipe *recipe;
+                g_autofree char *text = NULL;
+
+                recipe = gr_edit_page_get_recipe (GR_EDIT_PAGE (window->edit_page));
+                if (recipe)
+                        text = g_strdup_printf (_("The “%s” recipe has unsaved changes.\nClose the window anyway?"),
+                                                gr_recipe_get_name (recipe));
+                else
+                        text = g_strdup (_("The new recipe has not been saved.\nClose the window anyway?"));
+
+                dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                                 GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                 GTK_MESSAGE_QUESTION,
+                                                 GTK_BUTTONS_YES_NO,
+                                                 "%s", text);
+                g_signal_connect (dialog, "response", G_CALLBACK (response_cb), window);
+                gtk_widget_show (dialog);
+
+                return TRUE;
+        }
+
+        return FALSE;
+}
+
 static void
 hide_or_show_header_end_stack (GObject    *object,
                                GParamSpec *pspec,
@@ -502,7 +595,7 @@ do_undo (GrWindow *window)
 
         recipe = g_object_ref (window->undo_recipe);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         gr_recipe_store_add_recipe (store, window->undo_recipe, NULL);
         close_undo (window);
@@ -634,6 +727,72 @@ shopping_title_changed (GrWindow *window)
         }
 }
 
+static void
+close_shopping_done (GrWindow *window)
+{
+        if (window->shopping_done_timeout_id) {
+                g_source_remove (window->shopping_done_timeout_id);
+                window->shopping_done_timeout_id = 0;
+        }
+
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->shopping_done_revealer), FALSE);
+}
+
+static gboolean
+shopping_done_timeout (gpointer data)
+{
+        GrWindow *window = data;
+
+        close_shopping_done (window);
+
+        return G_SOURCE_REMOVE;
+}
+
+static void
+gr_window_offer_cooking (GrWindow *window)
+{
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->shopping_done_revealer), TRUE);
+        window->shopping_done_timeout_id = g_timeout_add_seconds (10, shopping_done_timeout, window);
+}
+
+static void gr_window_show_transient_list (GrWindow   *window,
+                                           const char *title,
+                                           GList      *recipes);
+
+static void
+done_shopping (GrWindow *window)
+{
+        GList *recipes;
+        GrRecipeStore *store;
+
+        recipes = gr_shopping_page_get_recipes (GR_SHOPPING_PAGE (window->shopping_page));
+
+        if (recipes->next == NULL)
+                gr_window_show_recipe (window, recipes->data);
+        else
+                gr_window_show_transient_list (window, _("Ready to Cook!"), recipes);
+
+        gr_window_offer_cooking (window);
+
+        g_list_free_full (recipes, g_object_unref);
+
+        store = gr_recipe_store_get ();
+        gr_recipe_store_clear_shopping_list (store);
+}
+
+static
+void make_save_sensitive (GrEditPage *edit_page, GParamSpec *pspec, gpointer data)
+{
+        GrWindow *window = GR_WINDOW (data);
+        gboolean unsaved;
+
+        g_object_get (G_OBJECT (edit_page), "unsaved", &unsaved, NULL);
+        if (unsaved)
+                gtk_widget_set_sensitive (window->save_button,TRUE);
+        else
+                gtk_widget_set_sensitive (window->save_button,FALSE);
+}
+
 static void
 gr_window_class_init (GrWindowClass *klass)
 {
@@ -651,12 +810,14 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrWindow, back_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, cooking_button);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, save_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_bar);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, main_stack);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, recipes_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, details_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, edit_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, list_page);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, transient_list_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, chef_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_page);
@@ -669,6 +830,7 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrWindow, remind_revealer);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, remind_label);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_added_revealer);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_done_revealer);
 
         gtk_widget_class_bind_template_callback (widget_class, new_recipe);
         gtk_widget_class_bind_template_callback (widget_class, go_back);
@@ -679,10 +841,11 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, search_changed);
         gtk_widget_class_bind_template_callback (widget_class, stop_search);
         gtk_widget_class_bind_template_callback (widget_class, search_mode_changed);
-        gtk_widget_class_bind_template_callback (widget_class, window_keypress_handler);
         gtk_widget_class_bind_template_callback (widget_class, window_keypress_handler_before);
+        gtk_widget_class_bind_template_callback (widget_class, window_keypress_handler_after);
         gtk_widget_class_bind_template_callback (widget_class, window_buttonpress_handler);
         gtk_widget_class_bind_template_callback (widget_class, window_mapped_handler);
+        gtk_widget_class_bind_template_callback (widget_class, window_delete_handler);
         gtk_widget_class_bind_template_callback (widget_class, do_undo);
         gtk_widget_class_bind_template_callback (widget_class, close_undo);
         gtk_widget_class_bind_template_callback (widget_class, do_remind);
@@ -690,13 +853,65 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, do_shopping_list);
         gtk_widget_class_bind_template_callback (widget_class, close_shopping_added);
         gtk_widget_class_bind_template_callback (widget_class, shopping_title_changed);
+        gtk_widget_class_bind_template_callback (widget_class, done_shopping);
+        gtk_widget_class_bind_template_callback (widget_class, close_shopping_done);
+        gtk_widget_class_bind_template_callback (widget_class, make_save_sensitive);
 }
 
+static GtkClipboard *
+get_clipboard (GtkWidget *widget)
+{
+        return gtk_widget_get_clipboard (widget, gdk_atom_intern_static_string ("CLIPBOARD"));
+}
+
+static void
+copy_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+        GtkWindow *window = GTK_WINDOW (user_data);
+        GtkWidget *focus;
+
+        focus = gtk_window_get_focus (window);
+        if (GTK_IS_TEXT_VIEW (focus))
+                gtk_text_buffer_copy_clipboard (gtk_text_view_get_buffer (GTK_TEXT_VIEW (focus)),
+                                                get_clipboard (focus));
+        else if (GTK_IS_ENTRY (focus))
+                gtk_editable_copy_clipboard (GTK_EDITABLE (focus));
+}
+
+static void
+paste_activated (GSimpleAction *action,
+                 GVariant      *parameter,
+                 gpointer       user_data)
+{
+        GtkWindow *window = GTK_WINDOW (user_data);
+        GtkWidget *focus;
+
+        focus = gtk_window_get_focus (window);
+        if (GTK_IS_TEXT_VIEW (focus))
+                gtk_text_buffer_paste_clipboard (gtk_text_view_get_buffer (GTK_TEXT_VIEW (focus)),
+                                                 get_clipboard (focus),
+                                                 NULL,
+                                                 TRUE);
+        else if (GTK_IS_ENTRY (focus))
+                gtk_editable_paste_clipboard (GTK_EDITABLE (focus));
+}
+
+static GActionEntry entries[] = {
+        { "copy", copy_activated, NULL, NULL, NULL },
+        { "paste", paste_activated, NULL, NULL, NULL }
+};
+
 static void
 gr_window_init (GrWindow *self)
 {
         gtk_widget_init_template (GTK_WIDGET (self));
         self->back_entry_stack = g_queue_new ();
+
+        g_action_map_add_action_entries (G_ACTION_MAP (self),
+                                         entries, G_N_ELEMENTS (entries),
+                                         self);
 }
 
 void
@@ -744,6 +959,7 @@ gr_window_edit_recipe (GrWindow *window,
         save_back_entry (window);
 
         gr_edit_page_edit (GR_EDIT_PAGE (window->edit_page), recipe);
+        gtk_widget_grab_focus (window->edit_page);
 
         gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_recipe_get_translated_name (recipe));
 
@@ -752,6 +968,7 @@ gr_window_edit_recipe (GrWindow *window,
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "edit");
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "edit");
+        gtk_widget_set_sensitive(window->save_button,FALSE);
 }
 
 static void
@@ -760,7 +977,7 @@ done_cb (GrRecipeImporter *importer,
          GrWindow         *window)
 {
         if (recipes)
-                gr_window_show_list (window, _("Imported Recipes"), recipes);
+                gr_window_show_transient_list (window, _("Imported Recipes"), recipes);
 }
 
 static void
@@ -1014,6 +1231,24 @@ gr_window_show_list (GrWindow   *window,
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "list");
 }
 
+static void
+gr_window_show_transient_list (GrWindow   *window,
+                               const char *title,
+                               GList      *recipes)
+{
+        save_back_entry (window);
+
+        gr_list_page_populate_from_list (GR_LIST_PAGE (window->transient_list_page), recipes);
+
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), title);
+
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "list");
+
+        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "transient");
+}
+
 void
 gr_window_show_shopping (GrWindow *window)
 {
@@ -1023,7 +1258,7 @@ gr_window_show_shopping (GrWindow *window)
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
-        gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "list");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "shopping");
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "shopping");
         gr_shopping_page_populate (GR_SHOPPING_PAGE (window->shopping_page));
@@ -1035,9 +1270,11 @@ do_show_myself (GrChef   *chef,
 {
         GrWindow *window = data;
 
+        save_back_entry (window);
+
         gr_list_page_populate_from_chef (GR_LIST_PAGE (window->chef_page), chef);
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header ), _("My own Recipes"));
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header ), _("My Recipes"));
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
@@ -1049,22 +1286,6 @@ do_show_myself (GrChef   *chef,
 void
 gr_window_show_myself (GrWindow *window)
 {
-        const char *name;
-        g_autoptr(GrChef) chef = NULL;
-        GrRecipeStore *store;
-
-        save_back_entry (window);
-
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
-        gr_ensure_user_chef (GTK_WINDOW (window), do_show_myself, window);
-
-        name = gr_recipe_store_get_user_key (store);
-        chef = gr_recipe_store_get_chef (store, name);
-        if (chef) {
-                do_show_myself (chef, window);
-                return;
-        }
-
         gr_ensure_user_chef (GTK_WINDOW (window), do_show_myself, window);
 }
 
@@ -1104,22 +1325,6 @@ gr_window_show_season (GrWindow   *window,
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "list");
 }
 
-void
-gr_window_show_search_by_ingredients (GrWindow   *window,
-                                      const char *ingredient)
-{
-        g_auto(GStrv) terms = NULL;
-
-        switch_to_search (window);
-        gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), TRUE);
-
-        terms = g_new (char *, 2);
-        terms[0] = g_strconcat ("i+:", ingredient, NULL);
-        terms[1] = NULL;
-
-        gr_query_editor_set_terms (GR_QUERY_EDITOR (window->search_bar), (const char **)terms);
-}
-
 void
 gr_window_show_image (GrWindow *window,
                       GArray   *images,
@@ -1129,6 +1334,7 @@ gr_window_show_image (GrWindow *window,
                 gr_image_page_set_images (GR_IMAGE_PAGE (window->image_page), images);
                 gr_image_page_show_image (GR_IMAGE_PAGE (window->image_page), index);
                 gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "image");
+                gtk_widget_grab_focus (window->image_page);
                 gr_window_set_fullscreen (window, TRUE);
         }
         else {
@@ -1187,6 +1393,18 @@ gr_window_show_about_dialog (GrWindow *window)
         gr_window_present_dialog (window, GTK_WINDOW (window->about_dialog));
 }
 
+static gboolean
+news_dialog_closed (GtkWidget *widget,
+                    GdkEvent  *event,
+                    gpointer   data)
+{
+        GrWindow *window = data;
+
+        window->news_dialog = NULL;
+
+        return FALSE;
+}
+
 void
 gr_window_show_news (GrWindow *window)
 {
@@ -1196,6 +1414,9 @@ gr_window_show_news (GrWindow *window)
         GtkWidget *box;
         int i;
 
+        if (window->news_dialog)
+                return;
+
         news = get_release_info (PACKAGE_VERSION, "0.0.0");
         if (news->len == 0)
                 return;
@@ -1203,6 +1424,13 @@ gr_window_show_news (GrWindow *window)
         builder = gtk_builder_new_from_resource ("/org/gnome/Recipes/recipe-whats-new-dialog.ui");
         dialog = GTK_WINDOW (gtk_builder_get_object (builder, "dialog"));
         gtk_window_set_transient_for (dialog, GTK_WINDOW (window));
+
+        window->news_dialog = (GtkWidget *)dialog;
+
+        gtk_widget_realize (GTK_WIDGET (dialog));
+        gdk_window_set_functions (gtk_widget_get_window (GTK_WIDGET (dialog)),
+                                  GDK_FUNC_ALL | GDK_FUNC_MINIMIZE | GDK_FUNC_MAXIMIZE);
+
         box = GTK_WIDGET (gtk_builder_get_object (builder, "box"));
 
         for (i = 0; i < news->len; i++) {
@@ -1251,6 +1479,7 @@ gr_window_show_news (GrWindow *window)
         }
 
         gtk_widget_show_all (box);
+        g_signal_connect (dialog, "delete-event", G_CALLBACK (news_dialog_closed), window);
         gr_window_present_dialog (GR_WINDOW (window), dialog);
 }
 
@@ -1260,8 +1489,6 @@ gr_window_show_report_issue (GrWindow *window)
         const char *uri = "https://bugzilla.gnome.org/enter_bug.cgi?product=recipes";
         g_autoptr(GError) error = NULL;
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Report Issue"));
-
         gtk_show_uri_on_window (GTK_WINDOW (window), uri, GDK_CURRENT_TIME, &error);
         if (error)
                 g_warning ("Unable to show '%s': %s", uri, error->message);
